@@ -2,9 +2,16 @@ import numpy as np
 from renderapi.errors import RenderError, EstimationError
 from renderapi.utils import encodeBase64, decodeBase64
 from .transform import Transform
+import scipy.spatial
+import logging
+import sys
 __all__ = ['ThinPlateSplineTransform']
 
 
+logger = logging.getLogger(__name__)
+logger.addHandler(logging.StreamHandler(sys.stdout))
+
+
 class ThinPlateSplineTransform(Transform):
     """
     render-python class that can hold a dataString for
@@ -81,47 +88,39 @@ class ThinPlateSplineTransform(Transform):
         numpy.array
             a Nx2 array of x,y points after transformation
         """
-        result = []
-        for pt in points:
-            result.append(self.apply(pt))
+        return self.apply(points)
 
-        return np.array(result)
-
-    def apply(self, pt):
+    def apply(self, points):
         if not hasattr(self, 'dMtxDat'):
-            result = pt
-            return result
+            return points
 
-        result = pt + self.computeDeformationContribution(pt)
+        result = points + self.computeDeformationContribution(points)
         if self.aMtx is not None:
-            result += self.aMtx.dot(pt)
+            result += self.aMtx.dot(points.transpose()).transpose()
         if self.bVec is not None:
             result += self.bVec
 
         return result
 
-    def computeDeformationContribution(self, pt):
-        disp = np.linalg.norm(
-                self.srcPts -
-                pt.reshape(self.ndims, 1),
-                axis=0)
-        nrm = np.zeros_like(disp)
-        ind = disp > 1e-8
-        nrm[ind] = disp[ind] * disp[ind] * np.log(disp[ind])
-        result = (nrm * self.dMtxDat).sum(1)
-        return result
+    def computeDeformationContribution(self, points):
+        disp = scipy.spatial.distance.cdist(
+                points,
+                self.srcPts.transpose(),
+                metric='sqeuclidean')
+        disp *= np.ma.log(np.sqrt(disp)).filled(0.0)
+        return disp.dot(self.dMtxDat.transpose())
 
     def gradient_descent(
             self,
-            pt,
+            pts,
             gamma=1.0,
             precision=0.0001,
             max_iters=1000):
         """based on https://en.wikipedia.org/wiki/Gradient_descent#Python
         Parameters
         ----------
-        pt : numpy array
-            [x,y] point for estimating inverse
+        pts : numpy array
+            a Nx2 array of x,y points
         gamma : float
             step size is gamma fraction of current gradient
         precision : float
@@ -130,23 +129,23 @@ class ThinPlateSplineTransform(Transform):
             limit for iterations, error if reached
         Returns
         -------
-        cur_pt : numpy array
-            [x,y] point, estimated inverse of pt
+        cur_pts : numpy array
+            a Nx2 array of x,y points, estimated inverse of pt
         """
-        cur_pt = np.copy(pt)
-        prev_pt = np.copy(pt)
+        cur_pts = np.copy(pts)
+        prev_pts = np.copy(pts)
         step_size = 1
         iters = 0
         while (step_size > precision) & (iters < max_iters):
-            prev_pt[:] = cur_pt[:]
-            cur_pt -= gamma*(self.apply(prev_pt) - pt)
-            step_size = np.linalg.norm(cur_pt - prev_pt)
+            prev_pts[:, :] = cur_pts[:, :]
+            cur_pts -= gamma*(self.apply(prev_pts) - pts)
+            step_size = np.linalg.norm(cur_pts - prev_pts, axis=1).max()
             iters += 1
         if iters == max_iters:
             raise EstimationError(
                     'gradient descent for inversion of ThinPlateSpline '
                     'reached maximum iterations: %d' % max_iters)
-        return cur_pt
+        return cur_pts
 
     def inverse_tform(
             self,
@@ -170,15 +169,12 @@ class ThinPlateSplineTransform(Transform):
         numpy.array
             a Nx2 array of x,y points after inverse transformation
         """
-        newpts = []
-        for p in points:
-            npt = self.gradient_descent(
-                    p,
-                    gamma=gamma,
-                    precision=precision,
-                    max_iters=max_iters)
-            newpts.append(npt)
-        return np.array(newpts)
+        newpts = self.gradient_descent(
+                points,
+                gamma=gamma,
+                precision=precision,
+                max_iters=max_iters)
+        return newpts
 
     @staticmethod
     def fit(A, B, computeAffine=True):
@@ -291,3 +287,127 @@ class ThinPlateSplineTransform(Transform):
         b64_2 = encodeBase64(blk2)
 
         return '{} {} {}'.format(header, b64_1, b64_2)
+
+    @staticmethod
+    def mesh_refine(
+            new_src,
+            old_src,
+            old_dst,
+            old_tf=None,
+            computeAffine=True,
+            tol=1.0,
+            max_iter=50,
+            nworst=10,
+            niter=0):
+        """recursive kernel for adaptive_mesh_estimate()
+        Parameters
+        ----------
+        new_src : numpy.array
+            Nx2 array of new control source points. Adapts during recursion.
+            Seeded by adaptive_mesh_estimate.
+        old_src : numpy.array
+            Nx2 array of orignal control source points.
+        old_dst : numpy.array
+            Nx2 array of orignal control destination points.
+        old_tf : ThinPlateSplineTransform
+            transform constructed from old_src and old_dst, passed through
+            recursion iterations. Created if None.
+        computeAffine : boolean
+            whether returned transform will have aMtx
+        tol : float
+            in units of pixels, how close should the points match
+        max_iter: int
+            some limit on how many recursive attempts
+        nworst : int
+            per iteration, the nworst matching srcPts will be added
+        niter : int
+            passed through the recursion for stopping criteria
+
+        Returns
+        -------
+        ThinPlateSplineTransform
+        """
+
+        if old_tf is None:
+            old_tf = ThinPlateSplineTransform()
+            old_tf.estimate(old_src, old_dst, computeAffine=computeAffine)
+
+        new_tf = ThinPlateSplineTransform()
+        new_tf.estimate(
+                new_src,
+                old_tf.tform(new_src),
+                computeAffine=computeAffine)
+        new_dst = new_tf.tform(old_src)
+
+        delta = np.linalg.norm(new_dst - old_dst, axis=1)
+        ind = np.argwhere(delta > tol).flatten()
+
+        if ind.size == 0:
+            return new_tf
+
+        if niter == max_iter:
+            raise EstimationError(
+                    "Max number of iterations ({}) reached in"
+                    " ThinPlateSplineTransform.mesh_refine()".format(
+                        max_iter))
+
+        sortind = np.argsort(delta[ind])
+        new_src = np.vstack((new_src, old_src[ind[sortind[0: nworst]]]))
+
+        return ThinPlateSplineTransform.mesh_refine(
+            new_src,
+            old_src,
+            old_dst,
+            old_tf=old_tf,
+            computeAffine=computeAffine,
+            tol=tol,
+            max_iter=max_iter,
+            nworst=nworst,
+            niter=(niter + 1))
+
+    def adaptive_mesh_estimate(
+            self,
+            starting_grid=7,
+            computeAffine=True,
+            tol=1.0,
+            max_iter=50,
+            nworst=10):
+        """method for creating a transform with fewer control points
+        that matches the original transfom within some tolerance.
+        Parameters
+        ----------
+        starting_grid : int
+            estimate will start with an n x n grid
+        computeAffine : boolean
+            whether returned transform will have aMtx
+        tol : float
+            in units of pixels, how close should the points match
+        max_iter: int
+            some limit on how many recursive attempts
+        nworst : int
+            per iteration, the nworst matching srcPts will be added
+
+        Returns
+        -------
+        ThinPlateSplineTransform
+        """
+
+        mn = self.srcPts.min(axis=1)
+        mx = self.srcPts.max(axis=1)
+        xt, yt = np.meshgrid(
+                np.linspace(mn[0], mx[0], starting_grid),
+                np.linspace(mn[1], mx[1], starting_grid))
+        new_src = np.vstack((xt.flatten(), yt.flatten())).transpose()
+        old_src = self.srcPts.transpose()
+        old_dst = self.tform(old_src)
+
+        return ThinPlateSplineTransform.mesh_refine(
+                new_src,
+                old_src,
+                old_dst,
+                old_tf=self,
+                computeAffine=computeAffine,
+                tol=tol,
+                max_iter=max_iter,
+                nworst=nworst,
+                niter=0)
